"""

Test for attentionpoint suggestionlist objects

Meant to be subclassed

"""
from django.contrib.auth.models import User
from django.core import serializers
from django.test import TestCase

from installations import models as installations_models

from utils import tests as utils_tests
from utils.versioning.models import WorkVersion, Version

class APSuggestionlistTest(TestCase):
    fixtures = ['dummyusers', 'installations']
##    sc_model = Scenario
##    pl_model = ProtectionLayer
##    me_model = Measure
##    ap_model = AttentionPoint
##    apsl_model = APSuggestionlist
##    namespace = "degradations"

    def setUp(self):
        self.setPermissions()
        defaultuser = User.objects.get(pk=1)

        self.section = installations_models.Section.objects.filter(version__isnull=True)[0]
        self.scope = self.section.get_auto_scope()
        # make sure the scope is published
        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)
        self.scenario = self.sc_model(name='scenario', scope=self.scope)
        self.scenario.save(defaultuser)
        self.measure = self.me_model(name='PL', type='PL')
        self.measure.save(defaultuser)

        self.apsuggestionlist = self.apsl_model(name='SL')
        self.apsuggestionlist.save(defaultuser)
        self.slap = self.ap_model(name='sl attentionpoint', suggestionlist=self.apsuggestionlist)
        self.slap.save(defaultuser)
        self.emptysuggestionlist = self.apsl_model(name='empty SL')
        self.emptysuggestionlist.save(defaultuser)

    def test_model(self):
        # test scope and suggestionlist attributes
        self.assertEquals(self.slap.measure, None)
        self.assertEquals(self.slap.suggestionlist, self.apsuggestionlist)

    def test_model_delete(self):
        """
        test delete suggestionlist deletes all aps
        """
        apsl_c = self.apsl_model.objects.count()
        ap_c = self.ap_model.objects.count()

        defaultuser = User.objects.get(pk=1)
        suggestionlist = self.apsl_model(name='SL 2')
        suggestionlist.save(defaultuser)
        ap = self.ap_model(name='sl attentionpoint', suggestionlist=suggestionlist)
        ap.save(defaultuser)

        self.assertEquals(self.apsl_model.objects.count(), apsl_c+1)
        self.assertEquals(self.ap_model.objects.count(), ap_c+1)

        self.apsl_model.objects.get(pk=suggestionlist.pk).delete(defaultuser)

        self.assertEquals(self.apsl_model.objects.count(), apsl_c)
        self.assertEquals(self.ap_model.objects.count(), ap_c)

    def test_model_dump_suggestionlist(self):
        dump = self.apsuggestionlist.dump()
        objects=serializers.deserialize('json', dump)
        self.assertEquals(len(list(objects)), 2)

    def test_publish(self):
        defaultuser = User.objects.get(pk=1)

        apsl_c = self.apsl_model.objects.count()
        ap_c = self.ap_model.objects.count()
        self.assertNotEquals(self.apsl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertNotEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # publish
        v1 = Version(name='v1', published=True)
        v1.save(defaultuser)

        self.assertEquals(self.apsl_model.objects.count(), 2 * apsl_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c)
        self.assertEquals(self.apsl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # change something
        self.slap = self.ap_model.objects.get_versionedobject(v_id=self.slap.v_id, version=WorkVersion)
        self.slap.name = u'other name'
        self.slap.save(defaultuser)

        self.assertEquals(self.apsl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 1)

        v2 = Version(name='v2', published=True)
        v2.save(defaultuser, propagate=False)
        self.apsuggestionlist.publish(defaultuser, v2)

        self.assertEquals(self.apsl_model.objects.count(), 2 * apsl_c)
        self.assertEquals(self.ap_model.objects.count(), 2 * ap_c + 1)
        self.assertEquals(self.apsl_model.objects.filter(published_dirty=True).count(), 0)
        self.assertEquals(self.ap_model.objects.filter(published_dirty=True).count(), 0)

        # let's check the pl name in the published v2

        ap = self.ap_model.objects.get_versionedobject(v_id=self.slap.v_id, version=v2)
        self.assertEquals(ap.name, u'other name')

        # let's check the scenario name in the published v1

        ap = self.ap_model.objects.get_versionedobject(v_id=self.slap.v_id, version=v1)
        self.assertEquals(ap.name, u'sl attentionpoint')

    def test_apsuggestionlist_views(self):

        base_url = '/analyses/%s/apsuggestionlists/' % self.namespace
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3)

        class apsuggestionlists(utils_tests.tablelist_TestDefinition):
            description = 'list of attention point suggestionlists'
            url = base_url
            users = [1,2,]
            rowcount = 2
            contains = [{'url': base_url + 'add/', 'users': [1,]},]

        class apsuggestionlist_suggestionlist(utils_tests.TestDefinition):
            description = 'display apsuggestionlist 1 tab suggestionlist'
            url = base_url + '%s/suggestionlist/' % self.apsuggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.apsuggestionlist.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.apsuggestionlist.v_id, 'users': []},
            ]

        class apsuggestionlist_empty_suggestionlist(utils_tests.TestDefinition):
            description = 'display empty apsuggestionlist tab suggestionlist'
            url = base_url + '%s/suggestionlist/' % self.emptysuggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.emptysuggestionlist.v_id, 'users': [1,]},
                {'url': base_url + '%s/delete/' % self.emptysuggestionlist.v_id, 'users': [1,]},
            ]

        class apsuggestionlist_attentionpoints(utils_tests.TestDefinition):
            description = 'display apsuggestionlist tab attention points'
            url = base_url + '%s/attentionpoints/' % self.apsuggestionlist.v_id
            users = [1,2,]
            contains = [
                {'url': base_url + '%s/edit/' % self.apsuggestionlist.v_id, 'users': []},
                {'url': base_url + '%s/delete/' % self.apsuggestionlist.v_id, 'users': []},
            ]

        class apsuggestionlist_compare(utils_tests.TestDefinition):
            description = 'compare apsuggestionlist'
            url = base_url + '%s/compare/0/' % self.apsuggestionlist.pk
            users = [1,2,]

        class apsuggestionlist_report(utils_tests.edit_TestDefinition):
            description = 'pdf report for an apsuggestionlist'
            url = base_url + '%s/pdf/' % self.apsuggestionlist.v_id
            users = [1,2,]
            data = {
                "submit_pdf": "Generate pdf",
            }

        class apsuggestionlist_add(utils_tests.add_TestDefinition):
            description = 'add apsuggestionlist'
            url = base_url + 'add/'
            users = [1,]
            data = {
                "description": "",
                "management-referer": base_url
            }
            redirect_url = base_url
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.apsl_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class apsuggestionlist_edit(utils_tests.edit_TestDefinition):
            description = 'edit apsuggestionlist'
            url = base_url + '%s/edit/' % self.apsuggestionlist.v_id
            users = [1,]
            data = {
                "name": "apsuggestionlist",
                "description": "",
                "management-referer": base_url + '%s/suggestionlist/' % self.apsuggestionlist.v_id
            }
            redirect_url = base_url + '%s/suggestionlist/' % self.apsuggestionlist.v_id

        class apsuggestionlist_delete(utils_tests.delete_TestDefinition):
            description = 'delete apsuggestionlist'
            users = [1,]
            redirect_url = base_url
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = self.test_class.apsl_model(name='to_delete')
                newitem.save(defaultuser)
                self.pk = newitem.pk
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    self.test_class.apsl_model.objects.get(pk=self.pk).delete(defaultuser, override=True)

        class apsuggestionlists_for_pl_multiple(utils_tests.TestDefinition):
            description = 'display apsuggestionlists for protection layer multiple'
            url = '/analyses/%s/measures/%s/apsuggestionlists/' % (self.namespace, self.measure.v_id)
            users = [1,]
            rowcount = 2
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = self.test_class.apsl_model(name='second SL')
                newitem.save(defaultuser)
                self.pk = newitem.pk
            def posttest(self, testcase, allowed):
                # clean up
                self.test_class.apsl_model.objects.get(pk=self.pk).delete(defaultuser, override=True)

        class apsuggestionlists_for_pl_single(utils_tests.TestDefinition):
            description = 'display apsuggestionlists for protection layer single'
            url = '/analyses/%s/measures/%s/apsuggestionlists/' % (self.namespace, self.measure.v_id)
            users = [1,]
            redirect_url = '/analyses/%s/measures/1/apsuggestionlist/%s/' % (self.namespace, self.apsuggestionlist.v_id)

        class apsuggestionlist_for_pl(utils_tests.edit_TestDefinition):
            description = 'display apsuggestionlist for measure'
            url = '/analyses/%s/measures/%s/apsuggestionlist/%s/' % (self.namespace, self.measure.v_id, self.apsuggestionlist.v_id)
            users = [1,]
            data = {
                "attentionpoints-TOTAL_FORMS": "1",
                "attentionpoints-INITIAL_FORMS": "1",
                "attentionpoints-MAX_NUM_FORMS": "0",
                "attentionpoints-0-select": True,
                "attentionpoints-0-id": self.slap.id,
                "management-referer": '/installations/measure/1/attentionpoints/'
            }
            redirect_url = '/analyses/%s/measures/%s/attentionpoints/' % (self.namespace, self.measure.v_id)
            def pretest(self, testcase, allowed):
                self.ap_c = self.test_class.ap_model.objects.count()
            def extra_tests(self, testcase, response, allowed):
                if allowed:
                    testcase.assertEquals(self.test_class.ap_model.objects.count(), self.ap_c + 1)
                else:
                    testcase.assertEquals(self.test_class.ap_model.objects.count(), self.ap_c)
            def posttest(self, testcase, allowed):
                # clean up
                if allowed:
                    self.test_class.ap_model.objects.order_by('-id')[0].delete(defaultuser, override=True)

        class attentionpoint_add(utils_tests.add_TestDefinition):
            description = 'add attentionpoint to apsuggestionlist'
            url = base_url + '%s/attentionpoints/add/' % self.apsuggestionlist.v_id
            users = [1,]
            data = {
                "name": "bogus",
                "type": "High pressure",
                "management-referer": base_url + '%s/attentionpoints/' % self.apsuggestionlist.v_id,
            }
            redirect_url = base_url + '%s/attentionpoints/' % self.apsuggestionlist.v_id
            def posttest(self, testcase, allowed):
                if allowed:
                    self.test_class.ap_model.objects.order_by('-id')[0].delete(testcase.user, override=True)

        class apsuggestionlist_publish(utils_tests.add_TestDefinition):
            description = 'publish suggestion list'
            url = base_url + '%s/publish/' % self.apsuggestionlist.v_id
            users = [1,]
            data = {
                "name": "public version apsuggestionlist",
                "management-referer": base_url
            }
            redirect_url = base_url

        tests = [
            apsuggestionlists(self),
            apsuggestionlist_suggestionlist(self),
            apsuggestionlist_empty_suggestionlist(self),
            apsuggestionlist_attentionpoints(self),
            apsuggestionlist_compare(self),
            apsuggestionlist_report(self),
            apsuggestionlist_add(self),
            apsuggestionlist_edit(self),
            apsuggestionlist_delete(self),
            apsuggestionlists_for_pl_multiple(self),
            apsuggestionlists_for_pl_single(self),
            apsuggestionlist_for_pl(self),
            attentionpoint_add(self),
            apsuggestionlist_publish(self),
        ]

        utils_tests.run_tests(self, users, tests)



