"""
ExplosionProtections app

App for explosionprotection analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "explosionprotection"
    tree_root_elementtype = "damage"
    analysistype =  _('Explosion protection')
    analysistype_short =  _('Explosion protection__')
    scenario_displaypermission = 'explosionprotection.display_explosionprotectionscenarios'
    scenario_editpermission = 'explosionprotection.edit_explosionprotectionscenarios'
    suggestionlist_displaypermission = 'explosionprotection.display_explosionprotectionsuggestionlists'
    suggestionlist_editpermission = 'explosionprotection.edit_explosionprotectionsuggestionlists'
    scenarios_url = 'explosionprotection_scenarios'
    suggestionlists_url = 'explosionprotection_suggestionlists'
    addurl = 'explosionprotection_scenario_add'
    consequence_label = _('Explosion damage')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': ExplosionProtectionSuggestionlist,
            'scenario': ExplosionProtectionScenario,
            'treeelement': ExplosionProtectionTreeElement,
            'event': ExplosionProtectionEvent,
            'protectionlayer': ExplosionProtectionProtectionLayer,
            'measure': ExplosionProtectionMeasure,
            'protectionlayer_measure': ExplosionProtectionPL_Measure,
            'attentionpoint': ExplosionProtectionAttentionPoint,
            'apsuggestionlist': ExplosionProtectionAPSuggestionlist,
        }

class ExplosionProtectionSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_explosionprotectionsuggestionlists", "May display explosionprotection suggestion lists"),
            ("edit_explosionprotectionsuggestionlists", "May edit explosionprotection suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ExplosionProtectionScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(ExplosionProtectionSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_explosionprotectionscenarios", "May display explosionprotection scenarios"),
            ("edit_explosionprotectionscenarios", "May edit explosionprotection scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class ExplosionProtectionTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(ExplosionProtectionScenario, verbose_name = _('Scenario'))

    appconfig = AppConfig


class ExplosionProtectionEvent(base_models.Event):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionProtectionMeasure(base_models.Measure):
    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionProtectionProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(ExplosionProtectionMeasure,
        verbose_name = _('Measures'),
        through="ExplosionProtectionPL_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionProtectionPL_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(ExplosionProtectionProtectionLayer)
    measure = versioning_models.VersionedForeignKey(ExplosionProtectionMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ExplosionProtectionAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ExplosionProtectionAttentionPoint(base_models.AttentionPoint):
##    protectionlayer = versioning_models.VersionedForeignKey(ExplosionProtectionProtectionLayer,
##        verbose_name = _('Protection layer'), blank=True, null=True)
    measure = versioning_models.VersionedForeignKey(ExplosionProtectionMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(ExplosionProtectionAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission


make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

restore_version = curry(base_models.restore_version, appconfig=AppConfig)
versioning_models.restore_version.connect(restore_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 70)

class ExplosionProtection_SuggestionlistSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionsuggestionlists'
    fields = ['name', 'description']
    model = ExplosionProtectionSuggestionlist
    name = _('Suggestion list')

searches.register(ExplosionProtection_SuggestionlistSearch)

class ExplosionProtection_ScenarioSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionscenarios'
    fields = ['name', 'description', 'evaluation']
    model = ExplosionProtectionScenario
    name = _('Scenario')

searches.register(ExplosionProtection_ScenarioSearch)

class ExplosionProtection_EventSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionscenarios'
    fields = ['name', 'description']
    model = ExplosionProtectionEvent
    name = _('Cause')

searches.register(ExplosionProtection_EventSearch)

class ExplosionProtection_ProtectionLayerSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionscenarios'
    fields = ['name', 'description']
    model = ExplosionProtectionProtectionLayer
    name = _('Protection layer')

searches.register(ExplosionProtection_ProtectionLayerSearch)

class ExplosionProtection_MeasureSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionscenarios'
    fields = ['name', 'tag', 'description']
    model = ExplosionProtectionMeasure
    name = _('Measure')

searches.register(ExplosionProtection_MeasureSearch)

class ExplosionProtection_APSuggestionlistSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionsuggestionlists'
    fields = ['name', 'description']
    model = ExplosionProtectionAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(ExplosionProtection_APSuggestionlistSearch)

class ExplosionProtection_AttentionPointSearch(Search):
    permission = 'explosionprotection.display_explosionprotectionscenarios'
    fields = ['name', 'description']
    model = ExplosionProtectionAttentionPoint
    name = _('Attention point')

searches.register(ExplosionProtection_AttentionPointSearch)
