"""
Releases app

App for release analysis

"""
from analyses.analysisbase import models as base_models
from analyses.main import analysesmodules

from django.db import models
from django.db.models import signals as django_signals
from django.utils.functional import curry
from django.utils.translation import ugettext_lazy as _

from installations import models as installations_models

from utils.exchange import exchange
from utils.search.models import searches, Search
from utils.strings import add_suffix_to_string
from utils.trees import models as trees_models
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class AppConfig(base_models.AppConfig):
    namespace = "releases"
    tree_root_elementtype = "loc"
    analysistype =  _('Release limitation')
    analysistype_short =  _('Release limitation__')
    scenario_displaypermission = 'releases.display_releasescenarios'
    scenario_editpermission = 'releases.edit_releasescenarios'
    suggestionlist_displaypermission = 'releases.display_releasesuggestionlists'
    suggestionlist_editpermission = 'releases.edit_releasesuggestionlists'
    scenarios_url = 'releases_scenarios'
    suggestionlists_url = 'releases_suggestionlists'
    addurl = 'releases_scenario_add'
    consequence_label = _('Consequence of release')

    @classmethod
    def get_models(cls):
        return {
            'suggestionlist': ReleaseSuggestionlist,
            'scenario': ReleaseScenario,
            'treeelement': ReleaseTreeElement,
            'event': ReleaseEvent,
            'protectionlayer': ReleaseProtectionLayer,
            'measure': ReleaseMeasure,
            'protectionlayer_measure': ReleaseProtectionLayer_Measure,
            'attentionpoint': ReleaseAttentionPoint,
            'apsuggestionlist': ReleaseAPSuggestionlist,
        }

class ReleaseSuggestionlist(base_models.Suggestionlist):
    class Meta:
        permissions = (
            ("display_releasesuggestionlists", "May display release suggestion lists"),
            ("edit_releasesuggestionlists", "May edit release suggestion lists"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ReleaseScenario(base_models.Scenario):
    suggestionlist = versioning_models.VersionedForeignKey(ReleaseSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="scenarios")

    class Meta(base_models.Scenario.Meta):
        ordering = ('type', 'name',)
        permissions = (
            ("display_releasescenarios", "May display release scenarios"),
            ("edit_releasescenarios", "May edit release scenarios"),
        )

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

EVENT_ID = base_models.EVENT_ID
PROTECTIONLAYER_ID = base_models.PROTECTIONLAYER_ID

class ReleaseTreeElement(base_models.TreeElement):
    scenario = versioning_models.VersionedForeignKey(ReleaseScenario, verbose_name = _('Scenario'))
    appconfig = AppConfig


class ReleaseEvent(base_models.Event):

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ReleaseMeasure(base_models.Measure):

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ReleaseProtectionLayer(base_models.ProtectionLayer):
    measures = versioning_models.VersionedManyToManyField(ReleaseMeasure,
        verbose_name = _('Measures'),
        through="ReleaseProtectionLayer_Measure",
        blank=True, null=True,
        related_name='protectionlayers')

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ReleaseProtectionLayer_Measure(base_models.ProtectionLayer_Measure):
    protectionlayer = versioning_models.VersionedForeignKey(ReleaseProtectionLayer)
    measure = versioning_models.VersionedForeignKey(ReleaseMeasure)

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

class ReleaseAPSuggestionlist(base_models.APSuggestionlist):
    appconfig = AppConfig
    displaypermission = appconfig.suggestionlist_displaypermission
    editpermission = appconfig.suggestionlist_editpermission

class ReleaseAttentionPoint(base_models.AttentionPoint):
    measure = versioning_models.VersionedForeignKey(ReleaseMeasure, verbose_name = _('Measure'),
        blank=True, null=True,
        related_name="attentionpoints")
    suggestionlist = versioning_models.VersionedForeignKey(ReleaseAPSuggestionlist,
        verbose_name=_('Suggestion list'), blank=True, null=True,
        related_name="attentionpoints")

    appconfig = AppConfig
    displaypermission = appconfig.scenario_displaypermission
    editpermission = appconfig.scenario_editpermission

make_version = curry(base_models.make_version, appconfig=AppConfig)
versioning_models.post_version.connect(make_version, sender=versioning_models.Version)

restore_version = curry(base_models.restore_version, appconfig=AppConfig)
versioning_models.restore_version.connect(restore_version, sender=versioning_models.Version)

analysesmodules.register(AppConfig, 30)

class Releases_SuggestionlistSearch(Search):
    permission = 'releases.display_releasesuggestionlists'
    fields = ['name', 'description']
    model = ReleaseSuggestionlist
    name = _('Suggestion list')

searches.register(Releases_SuggestionlistSearch)

class Releases_ScenarioSearch(Search):
    permission = 'releases.display_releasescenarios'
    fields = ['name', 'description', 'evaluation']
    model = ReleaseScenario
    name = _('Scenario')

searches.register(Releases_ScenarioSearch)

class Releases_EventSearch(Search):
    permission = 'releases.display_releasescenarios'
    fields = ['name', 'description']
    model = ReleaseEvent
    name = _('Cause')

searches.register(Releases_EventSearch)

class Releases_ProtectionLayerSearch(Search):
    permission = 'releases.display_releasescenarios'
    fields = ['name', 'description']
    model = ReleaseProtectionLayer
    name = _('Protection layer')

searches.register(Releases_ProtectionLayerSearch)

class Releases_MeasureSearch(Search):
    permission = 'releases.display_releasescenarios'
    fields = ['name', 'tag', 'description']
    model = ReleaseMeasure
    name = _('Measure')

searches.register(Releases_MeasureSearch)

class Releases_APSuggestionlistSearch(Search):
    permission = 'releases.display_releasesuggestionlists'
    fields = ['name', 'description']
    model = ReleaseAPSuggestionlist
    name = _('Attention point suggestion list')

searches.register(Releases_APSuggestionlistSearch)

class Releases_AttentionPointSearch(Search):
    permission = 'releases.display_releasescenarios'
    fields = ['name', 'description']
    model = ReleaseAttentionPoint
    name = _('Attention point')

searches.register(Releases_AttentionPointSearch)

