# substance views

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from utils import crud
from utils import forms as utils_forms
from utils.autocomplete import autocomplete
from utils.versioning import versionedcrud
from utils.versioning.session import get_current_version
from utils.versioning.models import WorkVersion

from hazards import models as app_models
from hazards import forms as app_forms
from hazards import tables as app_tables
from hazards.views.main import hazards_navigationlist

from installations import models as installations_models
from installations import tables as installations_tables

class MixIn(object):
    "Common items for object views"
    model = app_models.Substance
    include_attachments = True
    include_links = True

    def add_to_extra_context(self, extra_context, object):
        extra_context['navigationlist'] =  hazards_navigationlist()

    tabs = [
        {'name': 'substance',
            'label': _('Properties')},
        {'name': 'reactions',
            'label': _('Reactions')},
        {'name': 'sections',
            'label': _('Sections')},
    ]

class display_substance(MixIn, versionedcrud.DisplayObjectView):
    form_class = app_forms.SubstanceSheet
    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)

class display_reactions(MixIn, versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        queryset = object.reaction_set.all().select_related_versioned()
        table = app_tables.ReactionsTable(queryset)
        extra_context['table'] = table

class display_sections(MixIn, versionedcrud.DisplayObjectView):
    allow_change = False
    allow_delete = False
    form_class = None

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.DisplayObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)
        queryset = object.section_set.all()
        table = installations_tables.SectionsTable(queryset)
        extra_context['table'] = table

class display(MixIn, crud.TabContainer):
    tabviews = {
        'substance': display_substance(),
        'reactions': display_reactions(),
        'sections': display_sections(),
    }
    default = 'substance'

class edit(MixIn, versionedcrud.EditObjectView):
    form_class = app_forms.SubstanceForm
    tabs = []

class delete(MixIn, versionedcrud.DeleteObjectView):
    form_class = app_forms.SubstanceSheet
    tabs = []

    def get_default_url(self, object):
        return reverse('hazards_substances')

class report(MixIn, versionedcrud.ReportObjectView):
    form_class = utils_forms.ReportForm

class add(MixIn, versionedcrud.AddObjectView):
    form_class = app_forms.SubstanceForm
    tabs = []

class compare(MixIn, versionedcrud.CompareObjectView):
    form_class = app_forms.SubstanceSheet

    def add_to_extra_context(self, extra_context, object):
        versionedcrud.CompareObjectView.add_to_extra_context(self,extra_context, object)
        MixIn().add_to_extra_context(extra_context, object)


def name_lookup(request):
    (version, special) = get_current_version(request)
    queryset = app_models.Substance.objects.get_versionedobjects(version=version).values('name').order_by('name')
    return autocomplete(request, queryset, 'name', 'name')

class SubstanceFilter(crud.Filter):
    form = app_forms.SubstanceFilterForm
    filters = [
                {'filteritem': 'name', 'parameter': 'name__icontains', 'type': 'normal'},
                {'filteritem': 'tag', 'parameter': 'tag__icontains', 'type': 'normal'},
            ]

class SubstanceListObject(versionedcrud.TableListObject):
    defaultsort = 'name'
    table_class = app_tables.SubstancesTable
    queryset = app_models.Substance.objects.all()
    title = _('Substances')

    def add_html(self, html_strings, queryset, user, language, choices):
        if choices.get('substances'):
            for item in queryset:
                html_strings.append('<pdf:nextpage>')
                html_strings.extend(item.get_report_body(user, language, choices))

    def add_css(self, css_files, queryset, user, language, choices):
        if choices.get('substances'):
            if queryset:
                css_files.extend(queryset[0].get_report_css(language, choices))

class substances(versionedcrud.TableListView):
    "List of substances"

    table_class = app_tables.SubstancesTable
    csv_table_class = app_tables.SubstancesCSVTable
    defaultsort = 'name'
    queryset = app_models.Substance.objects.all()
    paginate_by = 100
    title = _('Substances')
    filter = SubstanceFilter()
    redirect_on_unique = True
    allow_pdf = True
    allow_csv = True
    pdfselectform_class = utils_forms.ListReportForm
    pdfsendmail = True
    listobject = SubstanceListObject()

    def get_pdfselectform(self):
        if self.request.method == 'POST':
            form = self.get_pdfselectform_class()(data=self.request.POST, user=self.request.user)
        else:
            form = self.get_pdfselectform_class()(user=self.request.user)
        return form

    def get_pdfselectform_class(self):
        return utils_forms.create_class_with_meta(self.pdfselectform_class, model=app_models.Substance)

    def add_to_extra_context(self, extra_context):
        super(substances, self).add_to_extra_context(extra_context)
        extra_context['navigationlist'] =  hazards_navigationlist('substances', self.request.user)
        tablebuttons = []
        if app_models.may_publish(self.request.user):
            tablebuttons.append({
                'url': reverse('hazards_publish'),
                'label': _('Publish')}
            )
        if (self.version==WorkVersion) and app_models.Substance.may_add(self.request.user):
            tablebuttons.append({
                    'url': reverse('hazards_substance_add'),
                    'label': _('Add')}
            )
        extra_context['tablebuttons'] = tablebuttons

