from django import forms
"""
helper functions and views for lists

"""

from django.template import loader, RequestContext
from django.template import Parser, FilterExpression, Variable, Template
from django.template import TemplateSyntaxError, VariableDoesNotExist
from django.http import Http404, HttpResponse
from django.http import HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.db.models import Q
from django.core.paginator import Paginator, InvalidPage

# use of filter forms

def get_filter_choices(dataset, field=None, callable=None, include_empty=True):
    """
    Returns a list of choices from a dataset.
    The dataset can be a list, a tuple or a queryset.
    Use the field parameter to modify the display of the queryset row.
    Use include_empty to include an empty choice on top of the choiceset.
    """
    emptychoice = [('', '---------')]
    if include_empty:
        choices = emptychoice[:]
    else:
        choices = []
    if isinstance(dataset, (list, tuple)):
        choices.extend(dataset)
    else:
        if field:
            choices.extend([(row.id, '%s' % getattr(row,field)) for row in dataset])
        elif callable:
            choices.extend([(row.id, '%s' % getattr(row,callable)()) for row in dataset])
        else:
            choices.extend([(row.id, '%s' % row) for row in dataset])
    return choices

class FilterForm(forms.Form):
    """
    Custom class add specific css classes to filterforms
    """
    def as_div(self):
        """Returns this form rendered as HTML <div class="filteritem">s."""
        return self._html_output(u'<div class="filteritem">%(label)s %(field)s%(help_text)s%(errors)s</div>',
            u'<div class="filteritem">%s</div>', '</div>', u'<br />%s', False)
    def __unicode__(self):
        return self.as_div()
    def __init__(self, *args, **kwargs):
        super(FilterForm, self).__init__(*args, **kwargs)
        # add attrs to widgets
        if hasattr(self, 'Meta'):
            attrs = getattr(self.Meta, 'attrs', {})
            for field in attrs:
                try:
                    self.fields[field].widget.attrs.update(attrs[field])
                except:
                    pass

def params_from_dict(filters, dict, prefix=u''):
    """
    Converts post parameters in the dict into Q objects and parameters
    based on the provided filters
    """
    Qobjs = []
    params = {}
    pref = prefix and (prefix + u'-') or u''
    for filter in filters:
        item = dict.get(pref+filter['filteritem'], None)
        if item is not None and item != u'':
            if filter['type'] == 'normal':
                if isinstance(item, list):
                    myQ = None
                    for c in item:
                        if myQ:
                            myQ = myQ | Q(**{filter['parameter']: c})
                        else:
                            myQ = Q(**{filter['parameter']: c})
                    if myQ:
                        Qobjs.append(myQ)
                else:
                    params[filter['parameter']] = item
            elif filter['type'] == 'boolean':
                params[filter['parameter']] = {u'True': True, u'False': False, True: True, False: False}[item]
            elif filter['type'] == 'function':
                myQobjs, myparams = filter['function'](item)
                Qobjs.extend(myQobjs)
                params.update(myparams)
    return Qobjs, params

def values_from_get(qdict, prefix=u''):
    """
    Helper function to resolve the querydict from request.GET into single values or lists
    """
    result = {}
    for k,v in qdict.lists():
        if prefix:
            key = k[len(prefix)+1:]
        else:
            key = k
        if len(v)==1:
            result[key] = v[0]
        else:
            result[key] = v
    return result


