"""
Tests for django-sheets app
"""

import datetime

from django import test
from django.conf import settings
from django.utils.encoding import smart_unicode, force_unicode
from django.utils.html import escape, conditional_escape
from django.utils.safestring import mark_safe

from utils import sheets

import utils.sheets.sheets_test.models as test_models

class test_sheets(test.TestCase):
    def setUp(self):
        s = test_models.Street(name='First')
        s.save()

        o1 = test_models.Owner(name='Washington')
        o1.save()
        o2 = test_models.Owner(name='Lincoln')
        o2.save()


        b = test_models.Building(number= 1, street=s, description='Basement\nAttic')
        b.save()
        b.owners.add(o1)
        b.owners.add(o2)

    def test_basefield(self):
        building = test_models.Building.objects.get(number=1)

        class DummySheet(sheets.Sheet):
            pass
        dummysheet = DummySheet()

        # normal case
        field = sheets.SheetField(name='number', data='number', label='Number')
        self.assertEqual(field.render_label(building), '<th>Number:</th>')
        self.assertEqual(field.render_field(building), '<td><div id="id_number">1</div></td>')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>')

        # test default for null values
        # without default
        field = sheets.SheetField(name='name', data='name', label='Name')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>')
        # with default
        field = sheets.SheetField(name='name', data='name', label='Name', default='White House')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Name:</th><td><div id="id_name">White House</div></td></tr>')

        # test callable data
        field = sheets.SheetField(name='number', label='Number',
            data = lambda x: x.number+2) # x is the instance, not the field value
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number">3</div></td></tr>')

        # test adding a link
        # fixed link
        field = sheets.SheetField(name='number', data='number', label='Number', link='someurl')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number"><a href="someurl">1</a></div></td></tr>')

        # callable link
        field = sheets.SheetField(name='number', data='number', label='Number',
            link = lambda x: 'someurl/%s' % x.number)
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number"><a href="someurl/1">1</a></div></td></tr>')

        # no label
        field = sheets.SheetField(name='number', data='number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>&nbsp;</th><td><div id="id_number">1</div></td></tr>')

        # label already has colon
        field = sheets.SheetField(name='number', data='number', label='Number:')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>')

        # field customisation
        # customised get_label
        class Field1(sheets.SheetField):
            def get_label(self, instance):
                return u'Custom label:'
        field = Field1(name='number', data='number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Custom label:</th><td><div id="id_number">1</div></td></tr>')

        # customised get_link
        class Field2(sheets.SheetField):
            def get_link(self, value, instance):
                return u'somespecialurl'
        field = Field2(name='number', data='number', label='Number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number"><a href="somespecialurl">1</a></div></td></tr>')

        # custom format_value
        class Field3(sheets.SheetField):
            def format_value(self, value):
                return mark_safe(u'<i>%s</i>' % conditional_escape(force_unicode(value)))
        field = Field3(name='number', data='number', label='Number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number"><i>1</i></div></td></tr>')

        # custom get_id
        class Field4(sheets.SheetField):
            def get_id(self, instance):
                return 'id_%s_special' % self.name
        field = Field4(name='number', data='number', label='Number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number_special">1</div></td></tr>')

        # custom get_field
        class Field5(sheets.SheetField):
            def get_field(self, instance):
                return 'x_%s_x' % instance.number
        field = Field5(name='number', data='number', label='Number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number">x_1_x</div></td></tr>')


        # test using a dictionary instead of a model instance
        item = {'number': 2}
        field = sheets.SheetField(name='number', data='number', label='Number')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Number:</th><td><div id="id_number">2</div></td></tr>')

    def test_specific_fields(self):
        building = test_models.Building.objects.get(number=1)

        class DummySheet(sheets.Sheet):
            pass
        dummysheet = DummySheet()

        # EmptyField
        field = sheets.EmptyField(name='number', label='Number')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Number:</th><td><div id="id_number">&nbsp;</div></td></tr>')

        # Textfield
        field = sheets.TextField(name='description', label='Description', data='description')
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Description:</th><td><div id="id_description"><p>Basement<br />Attic</p></div></td></tr>')

        # DateField
        item = {'date': datetime.date(2009,10,14)}
        field = sheets.DateField(name='date', label='Date', data='date', format='N j, Y')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Date:</th><td><div id="id_date">Oct. 14, 2009</div></td></tr>')
        field = sheets.DateField(name='date', label='Date', data='date', format='j/n/Y')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Date:</th><td><div id="id_date">14/10/2009</div></td></tr>')

        # TimeField
        item = {'time': datetime.time(10,10)}
        field = sheets.TimeField(name='time', label='Time', data='time', format='P')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Time:</th><td><div id="id_time">10:10 a.m.</div></td></tr>')
        field = sheets.TimeField(name='time', label='Time', data='time', format='H:i')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Time:</th><td><div id="id_time">10:10</div></td></tr>')

        # DateTimeField
        item = {'date': datetime.datetime(2009,10,14,10,10)}
        field = sheets.DateTimeField(name='date', label='Date', data='date', format='N j, Y, P')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Date:</th><td><div id="id_date">Oct. 14, 2009, 10:10 a.m.</div></td></tr>')
        field = sheets.DateTimeField(name='date', label='Date', data='date', format='j/n/Y H:i')
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Date:</th><td><div id="id_date">14/10/2009 10:10</div></td></tr>')

        # ChoicesField
        item = {'choice': 1}
        field = sheets.ChoicesField(name='choice', label='Choice', data='choice', choices=((1,'Cool'), (0,'Uncool')))
        self.assertEqual(field.render(dummysheet, item), '<tr><th>Choice:</th><td><div id="id_choice">Cool</div></td></tr>')

        # URLField
        field = sheets.URLField(name='url', label='URL', data='url')
        item = {'url': 'www.google.com'}
        self.assertEqual(field.render(dummysheet, item),
            '<tr><th>URL:</th><td><div id="id_url">http://www.google.com</div></td></tr>')
        item = {'url': 'http://www.google.com'}
        self.assertEqual(field.render(dummysheet, item),
            '<tr><th>URL:</th><td><div id="id_url">http://www.google.com</div></td></tr>')
        # link is optional via add_link, but defaults to the value itself
        field = sheets.URLField(name='url', label='URL', data='url', add_link=True)
        item = {'url': 'www.google.com'}
        self.assertEqual(field.render(dummysheet, item),
            '<tr><th>URL:</th><td><div id="id_url"><a href="http://www.google.com">http://www.google.com</a></div></td></tr>')


        # Foreignkeyfield
        # without link
        field = sheets.ForeignKeyField(name='street', label='Street', data='street')
        self.assertEqual(field.render(dummysheet, building), '<tr><th>Street:</th><td><div id="id_street">First</div></td></tr>')
        # with link
        field = sheets.ForeignKeyField(name='street', label='Street', data='street', add_link=True)
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Street:</th><td><div id="id_street"><a href="street_url">First</a></div></td></tr>')

        # ManyToManyField
        # normal
        field = sheets.ManyToManyField(name='owners', label='Owners', data='owners')
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Owners:</th><td><div id="id_owners">Washington, Lincoln</div></td></tr>')
        # newlines
        field = sheets.ManyToManyField(name='owners', label='Owners', data='owners', newlines=True)
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Owners:</th><td><div id="id_owners">Washington<br />Lincoln</div></td></tr>')
        # add_link
        field = sheets.ManyToManyField(name='owners', label='Owners', data='owners', add_link=True)
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Owners:</th><td><div id="id_owners"><a href="owner_url">Washington</a>, <a href="owner_url">Lincoln</a></div></td></tr>')
        # add_link and newlines
        field = sheets.ManyToManyField(name='owners', label='Owners', data='owners', add_link=True, newlines=True)
        self.assertEqual(field.render(dummysheet, building),
            '<tr><th>Owners:</th><td><div id="id_owners"><a href="owner_url">Washington</a><br />' +
            '<a href="owner_url">Lincoln</a></div></td></tr>')

        # FileField
# TODO add FileField test

    def test_sheets(self):
        building = test_models.Building.objects.get(number=1)

        # manual field definitions
        class BuildingManualSheet(sheets.Sheet):
            number = sheets.SheetField(name='number', label='Number', data='number')
            name = sheets.SheetField(name='name', label='Name', data='name')

        f = BuildingManualSheet(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>')


        # definitions from model
        class BuildingModelSheet1(sheets.Sheet):
            class Meta:
                model = test_models.Building

        f = BuildingModelSheet1(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>\n' +
            '<tr><th>Street:</th><td><div id="id_street">First</div></td></tr>\n' +
            '<tr><th>Description:</th><td><div id="id_description"><p>Basement<br />Attic</p></div></td></tr>\n' +
            '<tr><th>Owners:</th><td><div id="id_owners">Washington, Lincoln</div></td></tr>')


        # test fields
        class BuildingModelSheet2(sheets.Sheet):
            class Meta:
                model = test_models.Building
                fields = ['number', 'name']

        f = BuildingModelSheet2(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>')


        # test exclude
        class BuildingModelSheet3(sheets.Sheet):
            class Meta:
                model = test_models.Building
                exclude = ['name', 'owners']

        f = BuildingModelSheet3(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th>Street:</th><td><div id="id_street">First</div></td></tr>\n' +
            '<tr><th>Description:</th><td><div id="id_description"><p>Basement<br />Attic</p></div></td></tr>')


        # test links
        class BuildingModelSheet4(sheets.Sheet):
            class Meta:
                model = test_models.Building
                fields = ['number', 'street']
                links = ['street',]

        f = BuildingModelSheet4(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th>Street:</th><td><div id="id_street"><a href="street_url">First</a></div></td></tr>')

        # mix model and custom fields
        class BuildingModelSheet5(sheets.Sheet):
            number = sheets.EmptyField(name='number', label='Number')
            class Meta:
                model = test_models.Building

        f = BuildingModelSheet5(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th>Number:</th><td><div id="id_number">&nbsp;</div></td></tr>\n' +
            '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>\n' +
            '<tr><th>Street:</th><td><div id="id_street">First</div></td></tr>\n' +
            '<tr><th>Description:</th><td><div id="id_description"><p>Basement<br />Attic</p></div></td></tr>\n' +
            '<tr><th>Owners:</th><td><div id="id_owners">Washington, Lincoln</div></td></tr>')

        # Fieldsets
        # definitions from model
        class BuildingModelSheet6(sheets.Sheet):
            class Meta:
                model = test_models.Building
                fieldsets = [{'label': 'Building', 'fields': ['street', 'number']},{'label': 'Extra', 'fields': ['name', 'description']}]

        f = BuildingModelSheet6(instance=building)
        self.assertEqual(f.as_html(),
            '<tr><th colspan="2" class="fieldset">Building</th></tr>\n' +
            '<tr><th>Street:</th><td><div id="id_street">First</div></td></tr>\n' +
            '<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>\n' +
            '<tr><th colspan="2" class="fieldset">Extra</th></tr>\n' +
            '<tr><th>Name:</th><td><div id="id_name"></div></td></tr>\n' +
            '<tr><th>Description:</th><td><div id="id_description"><p>Basement<br />Attic</p></div></td></tr>')

        # test separate field rendering
        f = BuildingModelSheet1(instance=building)
        self.assertEqual(f['number'].__unicode__(),
            u'<tr><th>Number:</th><td><div id="id_number">1</div></td></tr>')
        self.assertEqual(f['number'].render_label(),
            u'<th>Number:</th>')
        self.assertEqual(f['number'].render_field(),
            u'<td><div id="id_number">1</div></td>')

