"""
Generic base tree models
"""
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import ugettext_lazy as _
from utils.versioning import models as versioning_models
from utils.versioning import versionedmptt

class TreeElement(versionedmptt.VersionedMpttModel):
    """
    generic treeelement
    """
    TreeElementChoices = ()
    elementtype = models.SmallIntegerField(choices=TreeElementChoices)
    content_type = models.ForeignKey(ContentType, blank=True, null=True, related_name="%(class)s_related")
    object_id = models.IntegerField(blank=True, null=True)
    content_object = versioning_models.VersionedGenericForeignKey('content_type', 'object_id')

    class Meta:
        abstract = True

    def _type_fk(self):
        return self.content_object

    def get_absolute_url(self):
        t = self._type_fk()
        if t:
            return t.get_absolute_url()
        return u''

    def __unicode__(self):
        t = self._type_fk()
        if t:
            return t.__unicode__()
        return u''

    def tree_display(self):
        t = self._type_fk()
        if t:
            return t.tree_display()
        return u'--'

    def tree_display_for_edit(self):
        t = self._type_fk()
        if t:
            return t.tree_display_for_edit()
        return u'--'

    def get_elementtype_for_tree(self):
        t = self._type_fk()
        if t:
            return t.get_elementtype_for_tree()
        return 'error'

    @classmethod
    def collect_items(cls, version):
        return list(('Tree', item, None) for item in cls.objects.filter(version=version, parent__isnull=True, deleted=False))

    def delete(self, user, *args, **kwargs):
        "remove leaf when deleting the last related treeelement"
        nocascade = kwargs.pop('nocascade', False)
        if not self.version:
            deleteleaf=False
            if not nocascade:
                co = self.content_object
                if co:
                    te = co.get_treeelements()
                    if len(te) == 1:
                        deleteleaf = True
            super(TreeElement, self).delete(user, *args, **kwargs)
            if deleteleaf and co:
                co.delete(user, raw=True, *args, **kwargs)
    delete.alters_data = True

    @classmethod
    def copy_tree(cls, tree, user, savekwargs={}, leafkwargs={}, **kwargs):
        """
        Copy an entire tree
        """
        newtree_id = cls.tree._get_next_tree_id()
        pkdict = {}
        for item in tree:
            oldv_id = item.v_id
            item.pk = None
            item.v_id = None
            item.tree_id = newtree_id
            item.dirty = True
            item.published_dirty = True
            item.published = False
            item.version = None
            if item.parent:
                item.parent = pkdict.get(item.parent.v_id)
            if kwargs:
                for prop in kwargs.keys():
                    setattr(item, prop, kwargs.get(prop))
            if item.content_object:
                item.content_object = item.content_object.copy(user, savekwargs={'save_treeelement': False}, **leafkwargs)
            versioning_models.VersionedModel.save(item, user, **savekwargs)
            pkdict[oldv_id]=item
        # return only ids
        res = {}
        for item in pkdict.keys():
            res[item] = pkdict.get(item).v_id
        return res

class TreeLeaf(versioning_models.VersionedModel):
    name = models.CharField(verbose_name=_('Name'), max_length=100)
    description = models.TextField(verbose_name = _('Description'), blank=True, null=True)

    class Meta:
        abstract = True
        ordering = ('name',)

    def __unicode__(self):
        return self.name

    def tree_display(self):
        return self.name

    def tree_display_for_edit(self):
        return self.name

    def get_elementtype_for_tree(self):
        return 'event'

    def create_treeelement(self, **kwargs):
        """
        Override this to create the treeelement in the appropriate tree
        no need to set content_object or parent
        """
        return TreeElement()

    def save(self, user, save_treeelement=True, *args, **kwargs):
        """When adding a new object, also create a tree element.
        Optionaly pass a parent kwarg to determine the parent element in the tree
        Returns the tree element
        """
        new = (not self.pk)
        treeparent = kwargs.pop('parent', None)
        treeelementkwargs = kwargs.pop('treeelementkwargs', {})
        super(TreeLeaf, self).save(user, *args, **kwargs)
        if new and save_treeelement:
            treeelement = self.create_treeelement(**treeelementkwargs)
            treeelement.content_object = self
            treeelement.parent = treeparent
            treeelement.save(user)
            return treeelement

    def get_tree_class(self):
        """
        Override to get the appropriate tree element class
        """
        return TreeElement

    def get_treeelements(self):
        version = getattr(self, '_version', self.version) or versioning_models.WorkVersion
        return self.get_tree_class().objects.filter(content_type=ContentType.objects.get_for_model(self.__class__),
             object_id=self.v_id).get_versionedobjects(version=version)

    def delete(self, user, raw=False, *args, **kwargs):
        "also set item in tree deleted"
        if self.version:
            raise Exception('Forbidden to delete versioned element')
        else:
            super(TreeLeaf, self).delete(user, *args, **kwargs)
            if not raw:
                sss = self.get_treeelements()
                for ss in sss:
                    ss.delete(user, nocascade=True, *args, **kwargs)

    def report_print_details(self):
        return bool(self.description)

